#!/bin/bash
# Autor: Vitalinux DGA
# Script que da servicio a las carpetas compartidas:
# * Como servicio de systemd
# * Checkea la conectividad para desmontar en caso de perder conexión y asegurar así la estabilidad del navegador de archivos
# Configurado en base al fichero de recursos nfs-recuros.json, personalizable por el paquete de centro correspondiente

crear_directorios_montaje() {
    # Se crean los directorios de montaje en función del usuario gráfico
    # Al montar sobre /media se creará un "shortcut" en el Escritorio de los usuarios
    dirmonta="$1"
    link_old="$2"
    local HOME
    HOME="${HOMEUSUGRAFICO}"
    variables_user_dirs="${HOME}/.config/user-dirs.dirs"

    [ -f "${variables_user_dirs}" ] && . ${variables_user_dirs}

    if [ ! -d "${dirmonta}" ]; then
        if ! mkdir -p "${dirmonta}"; then
            echo "Problemas al crear carpeta $dirmonta"
            return 1
        else
            echo "Se ha creado la carpeta: $dirmonta"

        fi
    else
        echo "El directorio \"${dirmonta}\" ya existe ... se montará encima de los datos existentes"
    fi
    # Crear link de compatibilidad
    [ "${link_old}" != "null" ] && ! [ "${link_old}" = "" ] && ! [ -L "${link_old}" ] && ln -sf "$dirmonta" "$link_old"

    #     # Desktop launcher
    #     if [ "${XDG_DESKTOP_DIR}" != "" ] && [ -d "${XDG_DESKTOP_DIR}" ]; then
    #         ${debug} && echo "=> Creando icono para ${dirmonta} en ${XDG_DESKTOP_DIR} ..."

    #         desktopname="${XDG_DESKTOP_DIR}/$(systemd-escape -p "${dirmonta}").desktop"
    #         cat >"${desktopname}" <<EOF
    # [Desktop Entry]
    # Type=Application
    # Exec=xdg-open "${dirmonta}"
    # Icon=folder-network
    # Name=$(systemd-escape -p ${dirmonta} | sed 's/media/srv/g')
    # EOF
    #         chown "${USUGRAFICO}" "${desktopname}"
    #         chmod +x "${desktopname}"
    #     else
    #         ${debug} && echo "=> No existe Escritorio ${XDG_DESKTOP_DIR} para crear icono de ${dirmonta}"
    #     fi
    return 0
}

check_cids_etiquetas_usuarios_grupos() {
    CIDSCENTRO="${1}"
    # De las etiquetas del centro eliminamos el sufijo ";" del final para evitar errores:
    ETIQUETASCENTRO="${2%;}"
    EXCLUIDOS="${3%;}"
    ALLOW_USERS="${4:-"null"}"
    ALLOW_GROUPS="${5:-"null"}"
    DENY_USERS="${6:-"null"}"
    DENY_GROUPS="${7:-"null"}"

    # Tenemos en cuenta la igualdad de cuentas: alumno|estudiante, profesor|docente
    [[ ";${ALLOW_USERS};" =~ \;alumno|estudiante\; ]] &&
        ALLOW_USERS="alumno;estudiante;${ALLOW_USERS}"
    [[ ";${ALLOW_USERS};" =~ \;profesor|docente\; ]] &&
        ALLOW_USERS="profesor;docente;${ALLOW_USERS}"
    [[ ";${DENY_USERS};" =~ \;alumno|estudiante\; ]] &&
        DENY_USERS="alumno;estudiante;${DENY_USERS}"
    [[ ";${DENY_USERS};" =~ \;profesor|docente\; ]] &&
        DENY_USERS="profesor;docente;${DENY_USERS}"

    # Chequeamos que el usuario gráfico esta incluido:
    if ! [[ "${ALLOW_USERS}" == "null" ||
        "${ALLOW_USERS}" == "all" ||
        ";${ALLOW_USERS};" =~ \;${USUGRAFICO}\; ]]; then
        ${debug} && echo "=> El equipo esta excluido para este usuario gráfico ..."
        return 1
    fi
    # Chequeamos si el usuario gráfico esta excluido:
    if [[ "${DENY_USERS}" != "null" && ";${DENY_USERS};" =~ \;${USUGRAFICO}\; ]] ||
        [[ "${DENY_USERS}" == "all" ]]; then
        ${debug} && echo "=> El usuario gráfico esta excluido explicitamente ..."
        return 1
    fi
    # Chequeamos que alguno de los grupos del usuario gráfico esta incluido:
    if ! [[ "${ALLOW_GROUPS}" == "null" ||
        "${ALLOW_GROUPS}" == "all" ]]; then
        EXCLUIR="true"
        for GRUPO in ${GRUPOSUSUGRAFICO}; do
            [[ ";${ALLOW_GROUPS};" =~ \;${GRUPO}\; ]] && EXCLUIR="false"
        done
        "${EXCLUIR}" && {
            ${debug} && echo "=> El equipo esta excluido por los grupos del usuario gráfico..."
            return 1
        }
    fi
    # Chequeamos si alguno de los grupos del usuario gráfico esta excluido:
    EXCLUIR="false"
    [[ "${DENY_GROUPS}" == "all" ]] && EXCLUIR="true"
    if [[ "${DENY_GROUPS}" != "null" ]]; then
        for GRUPO in ${GRUPOSUSUGRAFICO}; do
            [[ ";${DENY_GROUPS};" =~ \;${GRUPO}\; ]] && EXCLUIR="true"
        done
    fi
    "${EXCLUIR}" &&
        return 1

    # Comprobamos si el equipo esta incluido por CID o etiquetado Migasfree:
    if [[ ";${EXCLUIDOS};" =~ \;${MIGASFREECID}\; ||
        "${ETIQUETAS}" =~ ${EXCLUIDOS//;/|} ]]; then
        ${debug} && echo "=> El equipo (tag o cid) está excluido..."
        return 1
    fi

    if [[ ";${CIDSCENTRO};" =~ \;${MIGASFREECID}\;|all ]] ||
        [[ ";${ETIQUETASCENTRO};" =~ \;all\; ]] ||
        [[ "${ETIQUETAS}" =~ ${ETIQUETASCENTRO//;/|} ]]; then
        ${debug} && echo "=> El equipo esta incluido ..."
        return 0
    else
        ${debug} && echo "=> El equipo esta excluido (general)..."
        return 1
    fi
}

#echo "Inicio servicio de carpetas compartidas" | sudo vx-auditar-syslog

[[ "${VX_LOG_LEVEL}" = "debug" ]] && debug=true || debug=false

NFSRECURSOS="/usr/share/vitalinux/nfs-compartir/nfs-recursos.json"

if [ ! -f ${NFSRECURSOS} ]; then
    echo "No se encuentra el fichero de recursos...nada que hacer"
    exit 1
fi
# VARIABLES DE TRABAJO
# Variables de entorno
## Entorno de root ejecutado por usuario sudo
## Se añade como variables de entorno en el servicio el fichero de entorno

USUGRAFICO=$(vx-usuario-grafico)
GRUPOSUSUGRAFICO="$(id -Gn "${USUGRAFICO}")"
HOMEUSUGRAFICO=$(vx-home-usuario-grafico)
#MIGASFREECID -> Fichero de entorno cargado por systemd
#IPCACHE -> Fichero de entorno cargado por systemd

# Valorar si se metes en el bucle en el caso de que no se consigan las mismas!
if [ -f /tmp/migasfree.tags ]; then
    ETIQUETAS=$(</tmp/migasfree.tags)
else
    ETIQUETAS="$(migasfree-tags -g)"
fi

# Variable encargada de decidir si se muestran mensajes para evitar mensajes "pesados":
#  * Si en algún momento se pierde conexión con el servidor caché o no hay conexión se deshabilitan los mensajes
#  * Si hay un error en algún montaje también se deshabilitan los mensajes
MOSTRAR_MENSAJE=1

## RECURSOS A MONTAR
RECURSOSMONTAR=$(<"${NFSRECURSOS}")
SEC_COMPROBACION=$(jq -r '.sec_comprobacion' <<<"$RECURSOSMONTAR")
# Opciones de montaje por defecto:
MNTOPTIONSDFL=$(jq -r '.opciones_dfl' <<<"$RECURSOSMONTAR")

# En el caso de salir de forma inespearada desmontamos lo que haya montado
trap '/usr/bin/nfs-desmontajes.sh "${RECURSOSMONTAR}" "${HOMEUSUGRAFICO}"; exit' EXIT TERM INT QUIT

# DESMONTAR todos los recursos antes de iniciar
#  * Se evita así que haya recursos montados no pertenecientes al usuario
${debug} && echo "=> Intentamos desconectar recursos montados"
/usr/bin/nfs-desmontajes.sh "${RECURSOSMONTAR}" "${HOMEUSUGRAFICO}"
${debug} && echo "=> Desmontaje ok"

CONNECT=false
while true; do
    # Comprobamos si el servidor Caché esta presente y da servicio NFS cada sec_comprobacion por si hay caída
    ${debug} && echo "=> Test conexión $IPCACHE"
    if nc -z "${IPCACHE}" 2049 -w5 >/dev/null 2>&1; then
        CONNECT=true
        ${debug} && echo "=> Conexión ok...probando recursos"
        RECURSOSSERVIDOR=$(showmount -e "$IPCACHE" --no-headers | awk '{print $1 }')
        jq -c '.nfs_recursos.lista[] ' <<<"$RECURSOSMONTAR" | while read -r RECURSO; do
            CIDSREC=$(jq -r '.cids' <<<"$RECURSO")
            ETISREC=$(jq -r '.etiquetas' <<<"$RECURSO")
            EXCREC=$(jq -r '.excluidos' <<<"$RECURSO")
            ALLOW_USUREC=$(jq -r '.users' <<<"$RECURSO")
            ALLOW_GRPREC=$(jq -r '.groups' <<<"$RECURSO")
            DENY_USUREC=$(jq -r '.users_excluidos' <<<"$RECURSO")
            DENY_GRPREC=$(jq -r '.groups_exluidos' <<<"$RECURSO")
            RECURSOREMOTO=$(jq -r '.ruta' <<<"$RECURSO")

            ${debug} && echo "=> Comprobando permisos para Recurso $RECURSOREMOTO"
            if ! check_cids_etiquetas_usuarios_grupos "${CIDSREC}" "${ETISREC}" "${EXCREC}" "${ALLOW_USUREC}" "${ALLOW_GRPREC}" "${DENY_USUREC}" "${DENY_GRPREC}"; then
                # No procede montaje...
                continue
            fi
            CARPETAMONTAJE=$(jq -r '.puntomontaje' <<<"$RECURSO")
            LINKCOMPATIBILIDAD=$(jq -r '.link_compatibilidad' <<<"$RECURSO")
            MENSAJEOK=$(jq -r '.mensajeok' <<<"$RECURSO")
            MENSAJEERROR=$(jq -r '.mensajeerror' <<<"$RECURSO")

            # HAY OPCIONES DIFERENTES PARA EL RECURSO?
            [[ $(jq -r '.opciones' <<<"$RECURSO") = null ]] && MNTOPTIONS=$MNTOPTIONSDFL || MNTOPTIONS=$(jq -r '.opciones' <<<"$RECURSO")

            # ...comprobamos si estuviera montado y no debería
            grep "^${IPCACHE}:${RECURSOREMOTO}" /proc/mounts &>/dev/null
            MONTADO=$?
            grep "${RECURSOREMOTO}" &>/dev/null <<<"${RECURSOSSERVIDOR}"
            ACCESIBLE=$?

            if [ $MONTADO -eq 0 ] && [ $ACCESIBLE -ne 0 ]; then
                # Está montado y no debería Desmontamos y borramos acceso
                if umount -lf "${CARPETAMONTAJE}"; then
                    [ -z "$(ls -A "${CARPETAMONTAJE}")" ] && rmdir --ignore-fail-on-non-empty "${CARPETAMONTAJE}"
                    echo "Desmontando: $CARPETAMONTAJE...Algo ha pasado con el recurso compartido" &&
                        [ "${MOSTRAR_MENSAJE}" = "1" ] && vx-notifica-gui -i folder-network "Compartidas" "Algo ha pasado con un recurso compartido" &&
                        MOSTRAR_MENSAJE=0
                fi
                rm -f "$(vx-home-usuario-grafico)/Desktop/$(systemd-escape -p "${CARPETAMONTAJE}").desktop"
            elif [ $MONTADO -ne 0 ] && [ $ACCESIBLE -eq 0 ]; then
                if crear_directorios_montaje "${CARPETAMONTAJE}" "${LINKCOMPATIBILIDAD}" &&
                    mount -t nfs "${IPCACHE}:${RECURSOREMOTO}" "${CARPETAMONTAJE}" -o "${MNTOPTIONS}"; then
                    echo "Se monta el recurso ${IPCACHE}:${RECURSOREMOTO}" &&
                        [ "${MOSTRAR_MENSAJE}" = "1" ] && vx-notifica-gui -i folder-network "Compartidas" "${MENSAJEOK}"
                else
                    echo "Error al montar el recurso ${IPCACHE}:${RECURSOREMOTO}" &&
                        [ "${MOSTRAR_MENSAJE}" = "1" ] && vx-notifica-gui -i folder-network "Compartidas" "${MENSAJEERROR}" &&
                        MOSTRAR_MENSAJE=0

                fi
            fi
        done
    else
        MOSTRAR_MENSAJE=0
        ${CONNECT} && /usr/bin/nfs-desmontajes.sh "${RECURSOSMONTAR}" "${HOMEUSUGRAFICO}"
        CONNECT=false
    fi
    # Por si se produjera una desconexión inesperada, cada sec_comprobacion segundos lo revisamos
    sleep "${SEC_COMPROBACION}"

done
